/***********************************************
 * Large Assignment 1: L-System Lace
 * Name:    Jamini Sahu
 * E-mail:  jsahu@unm.edu
 * Course:      CS591, Computational Fabrication
 * Submitted:    2/14/2022
 *
 * This program includes several different L-System functions, using an LSystem class and a
 *  Flower helper file.  There are five main functions: abstract_(), botanical(), shapes(), roganBrown1(),
 *  and roganBrown2(), each corresponding to a task in the assignment. The botanical() function is designed
 *  to create a separate file for each color component plus a full drawing, so it will create 5 files. 
 *  RoganBrown1() will create the drawing used for the final cutout, and roganBrown2() will create the original
 *  design I had in mind for the task. There are helper functions named circle_(), wave(), and leaf() that use
 *  turtle movement to draw specific shapes.
 *
 * Running this program once will create the PDF files needed to process each laser cutout or sketch. The final
 *  image will show the shapes() function, as it is the only one that isn't used for the physical designs.
***********************************************/

import Turtle.*;
Turtle t;
LSystem l;
int length, petals, flowerColor, leafColor;
float weight;
import processing.pdf.*;

int[] colors = {#755BC4, #F59B2C, #F27FE7, #7FD2F2, #B94258};

void setup() {
  background(#FFFFFF);
  strokeWeight(1.5);
  t = new Turtle(this);
  l = new LSystem(50, 30);
  size(600, 600);
  length = 10;

  beginRecord(PDF, "Abstract.pdf"); 
  abstract_();
  endRecord();
  
  for (int i = 1; i <= 5; i++) {
    beginRecord(PDF, "Botanical_P" + i + ".pdf"); 
    botanical(i);
    endRecord();
  }
  
  beginRecord(PDF, "RoganBrown1.pdf"); 
  roganBrown1();
  endRecord();
  
  beginRecord(PDF, "RoganBrown2.pdf"); 
  roganBrown2();
  endRecord();
  
  clear();
  shapes();
}

// draws Sierpinski triangle by replacing each triangle with inner triangle
void abstract_() { 
  t.setY(height / 2 - 70);
  l = new LSystem(15 ,120);
  String[][] rule = {{"F", "F+R-F-R+F"},
                     {"R", "RR"},
                     {"+", "+", "+", "+"},
                     {"[", "[", "[", "["},
                     {"]", "]", "]", "]"},
                     {"-", "-", "-", "-"}};
  l.setRule(rule);
  l.setAxiom("F+R+R");
  l.iterate(4);
  l.drawLSystem(t); 
  l.drawLSystem(t);  
  l.drawLSystem(t);
  t.left(60);
  l.drawLSystem(t);  
  l.drawLSystem(t);  
  l.drawLSystem(t);  
  println("done drawing!");
  
}

// draws flower bouquet
// if section == 5, draw full bouquet
// otherwise, draw the color component associated with each number 1-4
void botanical(int section) {
  String[][] rule = {{"X", "F[-X][+X]FX>", "F[-X]F[+X]FX>"},
                        {"F", "FF", "FF"},
                        {"+", "+", "+"},
                        {"[", "[", "["},
                        {"]", "][L]", "][L]"},
                        {"-", "-", "-"}};
                        
  String[][] leftVine = {{"X", "F[L]-F[L]-F[L]-F[L]F"},
                        {"L", "L", "L"}, 
                        {"F", "FF", "FF"},
                        {"+", "+", "+"},
                        {"[", "[", "["},
                        {"]", "]", "]"},
                        {"-", "-", "-"}};  
 String[][] rightVine = {{"X", "F[L]+F[L]+F[L]+F[L]F"},
                        {"L", "L", "L"}, 
                        {"F", "FF", "FF"},
                        {"+", "+", "+"},
                        {"[", "[", "["},
                        {"]", "]", "]"},
                        {"-", "-", "-"}};

  strokeWeight(4);
  l.setAxiom("X");
  l.angle = 18;
  l.size = 25;
  l.setRule(rule);
  l.iterate(3);

  println(section);
  if (section == 1){
  // do leaves + stems
  
    flowerColor = #FFFFFF;
    leafColor = #167621;
    stroke(#167621); 
    
    t.setY(height);
    t.setX(width/2);
    t.setHeading(0);
    petals = (int) random (5, 12);
    //l.iterate(3);
    length = 25;
    l.drawLSystem(t);  
      
    length = 20;
    l.size = 25;
    t.setY(height);
    t.setX(width/2 + 25);
    t.setHeading(-35);  
    petals = (int) random (5, 12);
    l.drawLSystem(t);  
    
    t.setY(height);
    t.setX(width/2 - 25);
    t.setHeading(35);
    petals = (int) random (5, 12);
    l.drawLSystem(t);  
  
    stroke(#167621); 
    l.setAxiom("X");
    l.setRule(leftVine);
    t.setY(height);
    t.setX(width/2 + 25);
    t.setHeading(-40);
    l.iterate(2);
    l.drawLSystem(t); 
  
    l.setAxiom("X");
    l.setRule(rightVine);
    t.setY(height);
    t.setX(width/2 - 25);
    t.setHeading(40);
    l.iterate(2);
    l.drawLSystem(t); 
  }
  else if (section == 2) {
  // do middle flowers
    stroke(#FFFFFF);  
    flowerColor = #7FD2F2;
    leafColor = #FFFFFF;
    t.setY(height);
    t.setX(width/2);
    t.setHeading(0);
    petals = (int) random (5, 12);
    length = 25;
    l.drawLSystem(t);  
}
  else if (section == 3) {
  // do left flowers
    stroke(#FFFFFF);
    flowerColor = #F59B2C;
    leafColor = #FFFFFF;
    length = 20;
    l.size = 25;
    t.setY(height);
    t.setX(width/2 + 25);
    t.setHeading(-35);  
    petals = (int) random (5, 12);
    l.drawLSystem(t);  

  } 
  else if (section == 4) {
  // do right flowers
    stroke(#FFFFFF);
    flowerColor = #F27FE7;
    leafColor = #FFFFFF;
    length = 20;
    l.size = 25;
    t.setY(height);
    t.setX(width/2 - 25);
    t.setHeading(35);
    petals = (int) random (5, 12);
    l.drawLSystem(t);  
  }
  else if (section == 5) {
  // do all of it
    flowerColor = #FFFFFF;
    leafColor = #167621;
    stroke(#167621); 
    
    flowerColor = #7FD2F2;
    t.setY(height);
    t.setX(width/2);
    t.setHeading(0);
    petals = (int) random (5, 12);
    //l.iterate(3);
    length = 25;
    l.drawLSystem(t);  

    stroke(#167621); 
    flowerColor = #F59B2C;
    length = 20;
    l.size = 25;
    t.setY(height);
    t.setX(width/2 + 25);
    t.setHeading(-35);  
    petals = (int) random (5, 12);
    l.drawLSystem(t);  
    
    stroke(#167621); 
    flowerColor = #F27FE7;
    t.setY(height);
    t.setX(width/2 - 25);
    t.setHeading(35);
    petals = (int) random (5, 12);
    l.drawLSystem(t);  
  
    stroke(#167621); 
    l.setAxiom("X");
    l.setRule(leftVine);
    t.setY(height);
    t.setX(width/2 + 25);
    t.setHeading(-40);
    l.iterate(2);
    l.drawLSystem(t); 
  
    l.setAxiom("X");
    l.setRule(rightVine);
    t.setY(height);
    t.setX(width/2 - 25);
    t.setHeading(40);
    l.iterate(2);
    l.drawLSystem(t);  
  }
  println("done drawing!");
}

// draws circular L-system
// each iteration draws four circles inside an outer, parent circle
void shapes() {
  stroke(#5B6BD1);
  String[][] rule = {{"X", "QcQcQcQc"}, //{"X", "X"},
                      {"-", "-"}, 
                      {"+", "+"}, 
                      {"|", "|"}, 
                      {"b", "b"}, 
                      {"s", "s"}, 
                      {"c", "scQcQcQcQb"}, 
                      {"Qc", "Qc"}, 
                      {"F", "F"}};
  l.size = 8;
  l.angle = 1;
  l.setRule(rule);
  l.setAxiom("X");  
  l.iterate(4);
  t.setX(width/4 - 100);
  
  //for(int i = 0; i < 1; i++) {

    l.drawLSystem(t);
    
  //}
  println("done drawing!");
}

// draws the circuler, wagon-wheel coral variety
void roganBrown1() {
  background(#FFFFFF);
  strokeWeight(1.5);  
  l = new LSystem(50,30);
  stroke(#050000);
  
  String[][] circleCoral = {{"X", "mFB"}, //"F", "F[F]-F[F]-F[F]-F", "F[F]+F[F]+F[F]+F"},
                     {"UFFmB", "UFFmB", "UFFmB", "UFFmB"},
                     {"m", "m", "m"},
                     {"FB", "+FB", "+FB"},
                     {"B", "+FB", "+FB"},
                     {"[", "[", "[", "["},
                     {"]", "]", "]", "]"},
                     {"-", "-", "-", "-"}}; 
  

    l.setRule(circleCoral);
    l.setAxiom("X");
    l.angle = 36;
    l.iterate(10);
    t.setY(height/2 - 100);
    t.setX(width/2 - 100);
    
    for(int j = 1; j < 6; j++) {
      for(int i = 1; i < 6; i++) {
        t.setY(i * 100);
        t.setX(j * 100);        
        if (l.size <= 55) 
          l.size += (int) random (0, 20);
        else if (l.size > 55) 
          l.size -= (int) random (0, 20);
        l.drawLSystem(t);
        
      }
      t.setY(height/2 + 100);
      t.setX(width/2 + 100);
    }
  
}

// draws another version of Rogan Brown-inspired art with three different systems
void roganBrown2() {
  background(#FFFFFF);
  strokeWeight(1.5);  
  //l = new LSystem(50,30);
  l.size = 50;
  l.angle = 30;
  stroke(#050000);
  
  String[][] circleCoral = {{"X", "mFB"}, 
                     {"UFFmB", "UFFmB", "UFFmB", "UFFmB"},
                     {"m", "m", "m"},
                     {"FB", "+FB", "+FB"},
                     {"B", "+FB", "+FB"},
                     {"[", "[", "[", "["},
                     {"]", "]", "]", "]"},
                     {"-", "-", "-", "-"}}; 

  String[][] straight = {{"F",  "FF[---FF+F+Fx][+++FF-F-Fx]|<"}, 
                     {"X", "X", "X"},
                     {"F", "F"},
                     {"|", "|"},
                     {"<", "<"}};

  String[][] squiggle = {{"X", "[-Xwwx][Xwwx][+Xwwx]w"}, 
                     {"X", "X", "X", "X"},
                     {"w", "wXw", "wXw", "w"},
                     {"x", "x", "w", "w"},
                     {"-", "-", "-", "-"},
                     {"+", "+", "-", "-"},
                     {"[", "[", "[", "["},
                     {"]", "]", "]", "]"},
                     {"<", "<", "<", "<"}};
  

    t.setY(150);
    t.setX(300);
    l.setRule(circleCoral);
    l.setAxiom("X");
    l.iterate(6);

    t.setY(55);
    t.setX(230);
    l.drawLSystem(t);
    
    l.size = l.size - 10;
    t.setY(100);
    t.setX(145);
    l.drawLSystem(t);
    
    l.size = l.size + 20;
    t.setY(200);
    t.setX(125);
    l.drawLSystem(t);

    l.size = l.size - 30;
    t.setY(295);
    t.setX(100);
    l.drawLSystem(t);
    
    t.setY(295);
    t.setX(165);
    l.drawLSystem(t);

    l.size = l.size + 20;
    t.setY(378);
    t.setX(125);
    l.drawLSystem(t);
    

    l.setAxiom("X");
    l.angle = 36;
    l.iterate(10);
    t.setY(height/2 - 100);
    t.setX(width/2 - 100);
 
        
    strokeWeight(7);
    t.setHeading(0);
    t.setY(height - 20);
    t.setX(400);
    t.left(60);
    fill(#FFFFFF);
    l.setRule(squiggle);
    l.setAxiom("X");
    l.iterate(3);
    l.drawLSystem(t);
    
    l.size = 10;
    l.setAxiom("F");
    l.setRule(straight); 
    l.iterate(3);
    t.setY(height);
    strokeWeight(weight);  
    t.setHeading(5);    
    t.setX(200);
    l.drawLSystem(t);
    
    l.size = 10;   
    weight = 7;
    strokeWeight(weight);  
    t.setY(height + 150);
    t.setHeading(-7);  
    t.setX(600);
    l.drawLSystem(t);
  
}

// draws wave based on size factor 
// the bigger the factor, the shorter the wave
void wave(int factor) {
  for (int j = 0; j < 150; j++) {
      t.forward(1.0 / factor);
      t.right(0.75);
  }
  for (int j = 0; j < 150; j++) {
      t.forward(1.0 / factor);
      t.left(0.75);
  }  
}

// draws leaf based on size factor 
// the bigger the factor, the smaller the leaf
void leaf(int factor) {
  t.left(55);
  for (int j = 0; j < 150; j++) {
      t.forward(1.0 / factor);
      t.right(0.75);
  }
  t.right(68);
  for (int j = 0; j < 150; j++) {
      t.forward(1.0 / factor);
      t.right(0.75);
  }  
}

// draws circular shape of a given size iter
// used to make full circles (iter = 360) and quarter circles (iter = 90)
void circle_(int iter) {
  
  float c = l.size * 180;
  for (int j = 0; j < iter; j++) {
      t.forward(c/360);
      t.right(1);
  }
}
