// set up the foundation for an L-system
class LSystem {
  String axiom;
  String computedWord;
  String[][] rules;
  float size, angle;
  ArrayList<Float[]> lastPos = new ArrayList<Float[]>();
  int count = 0;
  
  // instantiates L-system
  LSystem(float size, float angle) {
    this.size = size;
    this.angle = angle;
    axiom = "X";
    computedWord = axiom;
  }
  
  // set  axiom to string
  void setAxiom(String axiom) {
    this.axiom = axiom;
    computedWord = axiom;
  }
  
  // set rules to string array
  void setRule(String[][] rule) {
    this.rules = rule;
  }
   
  // run rule replacement algorithm
  void iterate (int iterations) {
    String tempWord;
    println("starting word: " + computedWord);
    for(int i=0; i<iterations; i++) {
      tempWord = computedWord;
      computedWord = "";
      for(int j=0; j<tempWord.length();j++) { //loop through word
        char c = tempWord.charAt(j);
        for(int k=0; k<rules.length;k++) { //loop through rules
          if (c==rules[k][0].charAt(0)) {
            //println((int) random (1, rules[0].length));
            computedWord = computedWord + rules[k][(int) random (1, rules[0].length)];
          }
        }
      }
      println("iteration " + i +": " + computedWord);
    }
  }
  
  // draw L-system using turtle t
  // defines symbols needed to do different turtle actions needed for design patterns
  void drawLSystem(Turtle t) {
    char c;
    for (int i=0; i<computedWord.length(); i++) {
      c = computedWord.charAt(i);
      if (c == 'F') 
        t.forward(size); 
      else if (c == 'w')
        wave(7);
      else if (c == 'c')
        circle_(360);
      else if (c == 'Q')
        circle_(90);      
      else if (c == 'f') {
        t.forward(size/2);
      }      else if (c == 'B') {
        t.back(size);
      }
      else if (c == '+')
        t.right(angle);
      else if (c == '-')
        t.left(angle);
      else if (c == '|') {
        if (size > 1) 
          size = size - 0.2;
        angle += 1;
      }
      else if (c == 'm')  {
        noFill();
        strokeWeight(10);
        ellipse(t.getX(), t.getY(), size * 2, size * 2);
        strokeWeight(1.5);
      }
      
      else if (c == 'x') {
        t.penDown();
       if (size < 1) 
         break;
       ellipse(t.getX(), t.getY(), 10, 10);  
      }
      else if (c == 'U')
        t.penUp();
      else if (c == 'R')
        t.forward(size);
      else if (c == 'L') {
        t.penDown();
        stroke(leafColor);
        t.left(60); 
        leaf(5);
        t.left(120);
        leaf(5);
        t.left(60);
      }
      else if (c == '3') {
        stroke(#CE0A17);
        t.penDown();
        heart(4);  
      }
      else if (c == '>') {
        t.penDown();
        stroke(flowerColor);
        flower(petals);  
      }
      else if (c == '<') {
        if (weight >= 2.2) 
          weight = weight - 0.2;
        strokeWeight(weight);
      }
      else if (c == 's') {
        if (l.size >= 1) 
          l.size = l.size / 2;
      }
      else if (c == 'b') 
          l.size = l.size * 2;
      else if (c == '[') 
        t.push(); 
      else if (c == ']') 
         t.pop();
    }
  }
}
